"""Python library to connect deCONZ and Home Assistant to work together."""

import logging

from .deconzdevice import DeconzDevice

_LOGGER = logging.getLogger(__name__)


class DeconzLightBase(DeconzDevice):
    """deCONZ light base representation.

    Dresden Elektroniks documentation of lights in deCONZ
    http://dresden-elektronik.github.io/deconz-rest-doc/lights/
    """

    def __init__(self, deconz_id, device, async_set_state_callback):
        """Set initial information about light.

        Set async callback to set state of device.
        """
        self._async_set_state_callback = async_set_state_callback
        super().__init__(deconz_id, device)

    def async_update(self, event):
        """New event for light.

        Check that state is part of event.
        Signal that light has updated state.
        """
        self.update_attr(event.get('state', {}))
        super().async_update(event)

    def as_dict(self):
        """Callback for __dict__."""
        cdict = super().as_dict()
        if '_async_set_state_callback' in cdict:
            del cdict['_async_set_state_callback']
        return cdict

    @property
    def brightness(self):
        """Brightness of the light.

        Depending on the light type 0 might not mean visible "off"
        but minimum brightness.
        """
        return self._bri

    @property
    def hue(self):
        """Color hue of the light.

        The hue parameter in the HSV color model is between 0°-360°
        and is mapped to 0..65535 to get 16-bit resolution.
        """
        return self._hue

    @property
    def sat(self):
        """Color saturation of the light.

        There 0 means no color at all and 255 is the greatest saturation
        of the color.
        """
        return self._sat

    @property
    def ct(self):
        """Mired color temperature of the light. (2000K - 6500K)."""
        return self._ct

    @property
    def xy(self):
        """CIE xy color space coordinates as array [x, y] of real values (0..1)."""
        if self._xy != (None, None):
            self._x, self._y = self._xy

        if self._x is not None and self._y is not None:
            x = self._x
            if self._x > 1:
                x = self._x / 65555
            y = self._y
            if self._y > 1:
                y = self._y / 65555
            return (x, y)

        return None

    @property
    def colormode(self):
        """The current color mode of the light.

        hs - hue and saturation
        xy - CIE xy values
        ct - color temperature
        """
        return self._colormode

    @property
    def effect(self):
        """Effect of the light.

        none - no effect
        colorloop
        """
        return self._effect

    @property
    def reachable(self):
        """True if the light is reachable and accepts commands."""
        return self._reachable


class DeconzLight(DeconzLightBase):
    """deCONZ light representation.

    Dresden Elektroniks documentation of lights in deCONZ
    http://dresden-elektronik.github.io/deconz-rest-doc/lights/
    """

    def __init__(self, device_id, device, async_set_state_callback):
        """Set initial information about light.

        Set async callback to set state of device.
        """
        deconz_id = '/lights/' + device_id
        self._alert = device['state'].get('alert')
        self._bri = device['state'].get('bri')
        self._colormode = device['state'].get('colormode')
        self._ct = device['state'].get('ct')
        self._effect = device['state'].get('effect')
        self._hue = device['state'].get('hue')
        self._on = device['state'].get('on')
        self._reachable = device['state'].get('reachable')
        self._sat = device['state'].get('sat')
        self._xy = (None, None)
        self._x, self._y = device['state'].get('xy', (None, None))
        super().__init__(deconz_id, device, async_set_state_callback)

    async def async_set_state(self, data):
        """Set state of light.

        {
            "on": true,
            "bri": 180,
            "hue": 43680,
            "sat": 255,
            "transitiontime": 10
        }
        """
        field = self.deconz_id + '/state'
        await self._async_set_state_callback(field, data)

    @property
    def state(self):
        """True if the light is on."""
        return self._on

    @property
    def alert(self):
        """Temporary alert effect.

        Following values are possible:
        none - light is not performing an alert
        select - light is blinking a short time
        lselect - light is blinking a longer time
        """
        return self._alert



"""Python library to connect deCONZ and Home Assistant to work together."""

import logging
from pprint import pformat

_LOGGER = logging.getLogger(__name__)


class DeconzDevice:
    """deCONZ resource base representation.

    Dresden Elektroniks REST API documentation
    http://dresden-elektronik.github.io/deconz-rest-doc/
    """

    def __init__(self, deconz_id, device):
        """Set initial information common to all device types."""
        self._deconz_id = deconz_id
        self._etag = device.get('etag')
        self._manufacturername = device.get('manufacturername')
        self._modelid = device.get('modelid')
        self._name = device.get('name')
        self._swversion = device.get('swversion')
        self._type = device.get('type')
        self._uniqueid = device.get('uniqueid')
        self._async_callbacks = []
        _LOGGER.debug('%s created as \n%s', self._name, pformat(self.__dict__))

    def register_async_callback(self, async_callback):
        """Register callback for signalling.

        Callback will be called at the end of
        updating device information in self.async_update.
        """
        self._async_callbacks.append(async_callback)

    def remove_callback(self, callback):
        """Remove callback previously registered."""
        if callback in self._async_callbacks:
            self._async_callbacks.remove(callback)

    def update_attr(self, attr):
        """Update input attr in self.

        Return list of attributes with changed values.
        """
        changed_attr = []
        for key, value in attr.items():
            if value is None:
                continue
            if getattr(self, "_{0}".format(key), None) != value:
                changed_attr.append(key)
                self.__setattr__("_{0}".format(key), value)
                _LOGGER.debug('%s: update %s with %s', self.name, key, value)
        return changed_attr

    def async_update(self, event, reason={}):
        """Signal that a new event has been received.

        Reason is used to convey a message to upper implementation.
        """
        for async_signal_update in self._async_callbacks:
            async_signal_update(reason)

    def as_dict(self):
        """Callback for __dict__."""
        cdict = self.__dict__.copy()
        if '_callback' in cdict:
            del cdict['_callback']
        return cdict

    @property
    def deconz_id(self):
        """Id to call device over API e.g. /sensors/1."""
        return self._deconz_id

    @property
    def etag(self):
        """HTTP etag which changes on any action to the device."""
        return self._etag

    @property
    def manufacturer(self):
        """The manufacturer of the device."""
        return self._manufacturername

    @property
    def modelid(self):
        """An identifier unique to the product."""
        return self._modelid

    @property
    def name(self):
        """Name of the device."""
        return self._name

    @property
    def swversion(self):
        """Firmware version."""
        return self._swversion

    @property
    def type(self):
        """Human readable type of the device."""
        return self._type

    @property
    def uniqueid(self):
        """The current state of the device."""
        return self._uniqueid



"""Python library to connect deCONZ and Home Assistant to work together."""

import logging

from .light import DeconzLightBase

_LOGGER = logging.getLogger(__name__)


class DeconzGroup(DeconzLightBase):
    """deCONZ light group representation.

    Dresden Elektroniks documentation of light groups in deCONZ
    http://dresden-elektronik.github.io/deconz-rest-doc/groups/
    """

    def __init__(self, device_id, device, async_set_state_callback):
        """Set initial information about light group.

        Set callback to set state of device.
        """
        deconz_id = '/groups/' + device_id
        self._all_on = device['state'].get('all_on')
        self._any_on = device['state'].get('any_on')
        self._bri = device['action'].get('bri')
        self._class = device.get('class')
        self._colormode = device['action'].get('colormode')
        self._ct = device['action'].get('ct')
        self._devicemembership = device.get('devicemembership')
        self._effect = device['action'].get('effect')
        self._hidden = device.get('hidden')
        self._hue = device['action'].get('hue')
        self._id = device.get('id')
        self._lights = device.get('lights')
        self._lightsequence = device.get('lightsequence')
        self._multideviceids = device.get('multideviceids')
        self._on = device['action'].get('on')
        self._reachable = True
        self._sat = device['action'].get('sat')
        self._scenes = {}
        self._xy = (None, None)
        self._x, self._y = device['action'].get('xy', (None, None))
        super().__init__(deconz_id, device, async_set_state_callback)
        self.async_add_scenes(device.get('scenes'), async_set_state_callback)

    async def async_set_state(self, data):
        """Set state of light group.

        {
            "on": true,
            "bri": 180,
            "hue": 43680,
            "sat": 255,
            "transitiontime": 10
        }

        Also update local values of group since websockets doesn't.
        """
        field = self.deconz_id + '/action'
        await self._async_set_state_callback(field, data)
        self.async_update({'state': data})

    def as_dict(self):
        """Callback for __dict__."""
        cdict = super().as_dict()
        if '_scenes' in cdict:
            del cdict['_scenes']
        return cdict

    @property
    def state(self):
        """True if any light in light group is on."""
        return self._any_on

    @property
    def groupclass(self):
        """"""
        return self._class

    @property
    def all_on(self):
        """True if all lights in light group are on"""
        return self._all_on

    @property
    def devicemembership(self):
        """List of device ids (sensors) when group was created by a device."""
        return self._devicemembership

    @property
    def hidden(self):
        """Indicate the hidden status of the group.

        Has no effect at the gateway but apps can uses this to hide groups.
        """
        return self._hidden

    @property
    def id(self):
        """The id of the group."""
        return self._id

    @property
    def lights(self):
        """A list of all light ids of this group.

        Sequence is defined by the gateway.
        """
        return self._lights

    @property
    def lightsequence(self):
        """A list of light ids of this group that can be sorted by the user.

        Need not to contain all light ids of this group.
        """
        return self._lightsequence

    @property
    def multideviceids(self):
        """A list of light ids of this group.

        Subsequent ids from multidevices with multiple endpoints.
        """
        return self._multideviceids

    @property
    def scenes(self):
        """A list of scenes of the group."""
        return self._scenes

    def async_add_scenes(self, scenes, async_set_state_callback):
        """Add scenes belonging to group."""
        self._scenes = {
            scene['id']: DeconzScene(self, scene, async_set_state_callback)
            for scene in scenes
            if scene['id'] not in self._scenes
        }

    def update_color_state(self, light):
        """Sync color state with light."""
        x, y = light.xy or (None, None)
        self.async_update({
            'state': {
                'bri': light.brightness,
                'hue': light.hue,
                'sat': light.sat,
                'ct': light.ct,
                'x': x,
                'y': y,
                'colormode': light.colormode,
            },
        })


class DeconzScene:
    """deCONZ scene representation.

    Dresden Elektroniks documentation of scenes in deCONZ
    http://dresden-elektronik.github.io/deconz-rest-doc/scenes/
    """

    def __init__(self, group, scene, async_set_state_callback):
        """Set initial information about scene.

        Set callback to set state of device.
        """
        self._group_id = group.id
        self._group_name = group.name
        self._id = scene.get('id')
        self._name = scene.get('name')
        self._deconz_id = group.deconz_id + '/scenes/' + self._id
        self._async_set_state_callback = async_set_state_callback

    async def async_set_state(self, data):
        """Recall scene to group."""
        field = self._deconz_id + '/recall'
        await self._async_set_state_callback(field, data)

    @property
    def deconz_id(self):
        """Id to call scene over API e.g. /groups/1/scenes/1."""
        return self._deconz_id

    @property
    def full_name(self):
        """Full name."""
        return self._group_name + ' ' + self._name

    @property
    def id(self):
        """Scene ID from deCONZ."""
        return self._id

    @property
    def name(self):
        """Scene name."""
        return self._name

    def as_dict(self):
        """Callback for __dict__."""
        cdict = self.__dict__.copy()
        if '_async_set_state_callback' in cdict:
            del cdict['_async_set_state_callback']
        return cdict
